/*                                                       Google code jam 2k13.
   
   Programma che dovrebbe verificare lo stato di una partita del gioco tictactoe.*/

/* 
   Il programma è costruito mediante tre funzioni che leggono i caratteri su ogni riga, colonna o diagonale, 
   e chiamano una funzione ad hoc per contare le occorrenze, che vengono memorizzate in un vettore.
   Una quinta funzione viene chiamata per controllare se, su ogni traccia, sono presenti quattro simboli uguali,
   tenendo conto che il carattere 'T' vale come jolly.
   I controlli restituiscono valore 0 se l'esito è negativo.
   Viceversa, restituiscono 1 e stampano l'output se l'esito è positivo.
   A quel punto il programma termina.
   Con la stessa logica, una sesta funzione si incarica di verificare se siano ancora possibili mosse, solo nel caso in cui
   i test precedenti siano stati vani.
   Se anche quest'ultima evenienza non trova riscontro, la partita si è conclusa in parità.   
*/

#include<stdio.h>
#include<stdlib.h>
#include<string.h>

int c_riga(char tabella[4][4]);              /* Controlla le righe */
int c_colonna(char tabella[4][4]);           /* Controlla le colonne */
int c_diagonale(char tabella[4][4]);         /* Controlla le diagonali */
void contasegni(char sign, int *vet);        /* Controlla un segno ed aumenta il valore della corrispondente casella del vettore */
int findwinner(int vet[4]);                  /* Controlla la presenza di un vincitore dato il numero di simboli */
int cercapunti(char tabella[4][4]);          /* Controlla la presenza di mosse possibili se non sono stati trovati vincitori */

int main(int argc, char *argv[])
{
	FILE *fin;                            // Puntatore al file di input
	char tab[4][4]={{0}, {0}, {0}, {0}};  // Conterrà la partita da analizzare
	int ris = 0;                          // Sara usata come controllo dell'esito delle funzioni
	char buf[10];                         // Per acquisire il numero delle partite con "fgets"
	int i=0, j=0, k=1;                    // Perchè k conterà le partite	
	char segno=1;	                      // Sarà il singolo segno della tabella
	
	if(argc != 2)                         // Controllo di coerenza sul numero di argomenti
	{
		puts("ERRORE: devi inserire il percorso del file di input.");
		return -1;
	}

	fin = fopen(argv[1], "r");            // Apro il file di input	

	if(fin == NULL)                       // Controllo di coerenza sul file di input
	{
		puts("ERRORE: non posso aprire il file di input");
		return -1;
	}

	fgets(buf, sizeof(buf), fin);       // Per acquisire il numero di partite, ovvero la prima riga del file	
	printf("\nNumero di partite: %s\n", buf);	
	
	/* 
	   Il ciclo continua ad acquisire caratteri fintanto che la matrice non è completa.
	   Poi partono i controlli.
	   Se si è giunti alla fine del file, si esce dal programma.
	*/	
	while(k <= atoi(buf))	
	{	
		printf("Partita %i: ", k);
		k++;                                                      // Preparo per una nuova partita
		for(i=0; i<4; i++)
		{
			for(j=0; j<4; j++)
			{
				segno = fgetc(fin);                     // Gli "if" servono ad acquisire il carattere solo se mi interessa
				if(segno == EOF)		          // La condizione è solo per cautela
				{
					puts("\nPartite terminate. Grazie per aver partecipato al nostro gioco!\n");
					return 0;
				}
				if(segno == 'T' || segno == 'X' || segno == '0' || segno == '.')
					tab[i][j] = segno;
				else                                      // Questo per non lasciare buchi nella tabella
					j--;
			}
		}
		ris = c_riga(tab);                 // Gli "if" servono per evitare controlli inutili nel caso l'esito sia già accertato
		if (ris == 0)
			ris = c_colonna(tab);
		if (ris == 0)
			ris = c_diagonale(tab);
		if (ris == 0)
			ris = cercapunti(tab);
		if (ris == 0)
			puts("Pareggio.");	
	}	
	puts("\nPartite terminate. Grazie per aver partecipato al nostro gioco!\n");
	return 0;
}
	
	
/* Questa funzione scorre le righe e conta le ricorrenze di ciascun simbolo in ognuna di esse */
int c_riga(char tabella[4][4])
{
	/* Il vettore raccoglie il numero di simboli su ciascuna riga: num[0] sono le T
								       num[1] sono le X    
								       num[2] sono gli 0
								       num[3] sono i .	
	*/
	int num[4]={0};   
	int i=0, j=0, k=0;	
	int win=0;                             /* Segnala se esiste un vincitore */
	char segno;	
	for(i=0; i<4; i++)
	{
		for(k=0; k<4; k++)             /* Reinizializzo il vettore a zero per un nuovo ciclo */
		{
			num[k] = 0;
		}		
		for(j=0; j<4; j++)
		{
			segno = tabella[i][j];
			contasegni(segno, num);			
			if(j==3)
				win = findwinner(num);
			if(win!=0)              /* Se trovo il vincitore, perchè proseguire... */
				return win;
		}
	}
	return win;                             /* Se sono arrivato qui, win vale zero */
}


/* Ora la stessa cosa per le colonne */
int c_colonna(char tabella[4][4])
{
	int num[4]={0};   
	int i=0, j=0, k=0;	
	int win=0;                                      /* Segnala se esiste un vincitore */
	char segno;	
	for(j=0; j<4; j++)
	{
		for(k=0; k<4; k++)                      /* Reinizializzo il vettore a zero per un nuovo ciclo */
		{
			num[k]=0;
		}	
		for(i=0; i<4; i++)
		{
			segno = tabella[i][j];
			contasegni(segno, num);			
			if(i==3)
				win = findwinner(num);
			if(win!=0)			 /* Se trovo il vincitore, perchè proseguire... */
				return win;
		}	
	}
	return win;                                      /* Se sono arrivato qui, win vale zero */
}


/* ...e per le diagnoali. Con qualche modifica.*/
int c_diagonale(char tabella[4][4])
{
	int num[4]={0};   
	int i=0, j=0, k=0;
	int win=0;                                        /* Segnala se esiste un vincitore */	
	char segno;	
	while(j<4 && i<4)                                 /* Per la diagonale principale */
	{
		segno=tabella[i][j];
		contasegni(segno, num);			
		if(i==3 && j==3)
			win = findwinner(num);
		if(win!=0)                                 /* Se trovo il vincitore, perchè proseguire... */
			return win;
		i++;
		j++;
	}
	if (win==0)
	{
		i=0;                                       /* Reinizializzo per la diagonale secondaria */
		j=3;
		for(k=0; k<4; k++)                         /* Reinizializzo il vettore a zero per un nuovo ciclo */
		{
			num[k]=0;
		}	
		while(i<4 && j>=0)
		{
			segno=tabella[i][j];
			contasegni(segno, num);			
			if(i==3 && j==0)
				win = findwinner(num);      /* Se trovo il vincitore, perchè proseguire... */
			if(win!=0)
				return win;                          
			i++;
			j--;
		}
	}
	return win;                                          /* Se sono arrivato qui, win vale zero */
}

	
/* Infine la ricerca del vincitore e la stampa dei risultati */
int findwinner(int vet[4])
{
	/* Il vettore raccoglie il numero di simboli su ciascuna riga: vet[0] sono le T
								       vet[1] sono le X    
								       vet[2] sono gli 0
								       vet[3] sono i .	
	*/
	if(vet[0]+vet[1] == 4)
	{
		puts("Il vincitore e': X.");
		return 1;    /* Ho trovato un vincitore: nel main salterò tutti i controlli successivi */
	}
	if(vet[0]+vet[2] == 4)
	{
		puts("Il vincitore e': 0.");
		return 1;
	}
	return 0;            /* Significa che non ci sono vincitori */
}


/* La funzione che cerca possibili mosse */
int cercapunti(char tabella[4][4])
{
	int i=0, j=0;
	for(i=0; i<4; i++)
	{
		for(j=0; j<4; j++)
		{
			if(tabella[i][j] == '.')
			{	
				puts("Nessun vincitore. Sono possibili ulteriori mosse.");
				return 1;
			}
		}
	}
	return 0;         /* Significa che non sono neppure possibili ulteriori mosse */
}


/* La funzione che conta i segni mettendo il numero di occorrenze in un vettore.
   Usa il passaggio per valore per il carattere, quello per indirizzo per il vettore. */
void contasegni(char sign, int *vet)
{
	switch (sign)
		{
			case 'T': vet[0]++; break;
			case 'X': vet[1]++; break;
			case '0': vet[2]++; break;
			case '.': vet[3]++; break;
		}			
}

